package edu.georgetown.gucs.experiment;

import edu.georgetown.gucs.utility.Pair;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Evan B.
 * @since 6/18/14.
 */
public class ExperimentResults implements Comparable {

    //make sure to
    private int averageMScore = 0;
    private int averageEScore = 0;

    private Map<Integer, Integer> aggregateScores = new HashMap<>();

    List<Integer> eResults;
    List<Integer> mResults;

    TrialParameters trialParameters;

    /**
     * Constructor to initalize an empty trialParameters
     */
    ExperimentResults() {

        this.trialParameters = new TrialParameters();

    }


    ExperimentResults(TrialParameters trialParameters) {

        this.trialParameters = trialParameters;

    }

    public void setThreadName(String threadName) {
    }

    public void setDictionarySize(int dictSize) {

    }

    public void setMatcherResults(ArrayList<Integer> mResults) {

        this.mResults = (!mResults.isEmpty()) ? mResults : null;

    }

    public void setExactResults(ArrayList<Integer> eResults) {


        this.eResults = (!eResults.isEmpty()) ? eResults : null;

    }

    public void setMatcherScore(int averageScore) {

        averageMScore = averageScore;

    }

    public void setAggregateScores(Integer score) {

        if (!aggregateScores.containsKey(score)) {
            aggregateScores.put(score, 1);

        } else {

            Integer newScore = 1 + aggregateScores.get(score);
            //todo port this to java7
          //  aggregateScores.replace(score, newScore);
        }


    }

    public void setExactScore(int averageScore) {
        averageEScore = averageScore;
    }

    public String getMangler() {
        return trialParameters.getMangler();
    }

    public String getMatcherName() {
        return trialParameters.getMatcherName();
    }

    public int getAverageMatcherScore() {
        return averageMScore;
    }

    public int getAverageExactScore() {
        return averageEScore;
    }

    public String getFileTokenInfo() {
        return trialParameters.getFileToken().toString();
    }

    public Pair<Double, Double> getIdfRange() {
        return trialParameters.getIDFRange();
    }


    public String getTokenizers() {

        StringBuilder strBuf = new StringBuilder();
        for (String str : trialParameters.getTokenizerNames()) {
            strBuf.append(str).append(" ");
        }

        return strBuf.toString();
    }



    /**
     * @return returns an array list of aggregated scores
     */
    public List<Pair<Integer,Integer>> getAggregateListScores() {

        List<Pair<Integer,Integer>> tempAgg = new ArrayList<>();
                //aggregateScores.entrySet().stream().map(score -> new Pair<>(score.getKey(), score.getValue())).collect(Collectors.toList());

        for(Map.Entry<Integer,Integer> aggregate : aggregateScores.entrySet()) {

            tempAgg.add(new Pair<Integer,Integer>(aggregate.getKey(), aggregate.getValue()));

        }

        return tempAgg;


    }

    public void printTrialParams() {

        System.out.println("[Mangler] " + trialParameters.getMangler());

        System.out.println("[Matcher Name] " + trialParameters.getMatcherName());

        System.out.println("[Average Matcher Score] " + averageMScore);

        System.out.println("[Average Exact Score] " + averageEScore);

        System.out.println("[IDF Range] " + getIdfRange().getA() + "-" +
                getIdfRange().getB());


        for(String tokenizers : trialParameters.getTokenizerNames()) {
            System.out.println(tokenizers);
        }

        //System.out.println("\n\n --------------------------------------------------");

    }

    public void printAggregateScores() {

        printTrialParams();

        System.out.println("Individual Trial Scores");
        for (Map.Entry<Integer, Integer> score : aggregateScores.entrySet()) {

            System.out.println("\t" + score.getKey() + ":" + score.getValue());
        }


        System.out.println(" ----------------------------------------------------\n\n");

    }


    @Override
    public int compareTo(Object o) {
        return ((ExperimentResults) o).getMangler().compareTo(getMangler());
    }
}
