package edu.georgetown.gucs.fingerprinter;

import edu.georgetown.gucs.dictionary.Dictionary;
import edu.georgetown.gucs.utility.*;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import java.util.concurrent.RecursiveTask;

/**
 * @author Evan B.
 * @since 11/13/14.
 */
public class BulkFingerprinterWorker extends RecursiveTask<List<Fingerprint>> {

    Fingerprinter fingerprinter;
    ArrayList<File> files = new ArrayList<File>();
    Dictionary dict;
    Integer start;
    Integer end;
    String fingerprinterName;
    
    List<Fingerprint> fingerprintList = new ArrayList<Fingerprint>();

    BulkFingerprinterWorker(String fingerprinterName,
			    Dictionary dict,
			    ArrayList<File> fileList,
			    Integer low,
			    Integer high)  throws SdTextException {

        //    Global.debug("IncomingLister", fileLister.size());
	//System.err.println("New bulkfingerprintworker with list size: " + fileList.size() + " low: " + low			   + " high: " + high);

	//System.err.println("1");
        this.files = fileList;
	this.start = low;
	this.end = high;
	this.dict = dict;
	
	//System.err.println("2");
	this.fingerprinterName = fingerprinterName;
	//System.err.println("3");

	try {
	    //System.err.println("4");
	    this.fingerprinter = (Fingerprinter) ObjectBuilder.buildObject(fingerprinterName);
	}
	catch (SdTextException e){
	    throw e;
	}

	this.fingerprinter.setDictionary(dict);
	//System.err.println("5");
    }

    protected List<Fingerprint> compute() {

	//System.err.println("Call to compute");
	
	if (end - start <= Global.ForkJoinMinTasks){

	    //System.err.println("In non-recursive branch, creating fingerprints");
	    
	    for (int i = start; i <= end; i++){		

		try {
		    List<Fingerprint> fp = fingerprinter.computeFingerprint(files.get(i).getAbsolutePath());
		
		// The get(0) is an ugly hack that takes the first fingerprint only when the splitter is active
		fingerprintList.add(fp.get(0));
		}
		catch (SdTextException e){
		    // FIX - figure out how to pass exceptions back up here and below
		}
		
		
	    }
	    
	}
	else {
	    //System.err.println("In recursive branch, creating fingerprints");
	    
	    try {
		Integer mid = start + ((end - start) / 2);
		BulkFingerprinterWorker left = new BulkFingerprinterWorker (fingerprinterName,
									    dict,
									    files,
									    start,
									    mid);
		
		BulkFingerprinterWorker right = new BulkFingerprinterWorker (fingerprinterName,
									     dict,
									     files,
									     mid + 1,
									     end);
		left.fork();
		
		List<Fingerprint> rightAnswer = right.compute();
		fingerprintList.addAll(rightAnswer);
		List<Fingerprint> leftAnswer = left.join();	    
		fingerprintList.addAll(leftAnswer);	   
	    }
	    catch (SdTextException e){
		// FIX - figure out how to handle exceptions here, since I can't seem to pass them up
	    }
		
	}
	
        return fingerprintList;
    }
}

