package edu.georgetown.gucs.matcher;

import edu.georgetown.gucs.fingerprinter.Fingerprint;
import edu.georgetown.gucs.utility.*;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.RecursiveTask;

/**
 * @author Clay Shields
 * @since 8/4/15.
 */
public class BulkListMatcherWorker extends RecursiveTask<List<Pair<String, Integer>>> {

    private String fingerprintMatcherName;
    private FingerprintMatcher matcher;
    private List<Fingerprint> fingerprintList;
    private Fingerprint singleFingerprint;
    private Integer start;
    private Integer end;

    public BulkListMatcherWorker(String matcherName,
			     Fingerprint single,
			     List<Fingerprint> list,
			     Integer low,
			     Integer high) throws SdTextException{

	this.fingerprintMatcherName = matcherName;
        this.start = low;
	this.end = high;
	this.singleFingerprint = single;
        this.fingerprintList = list;
	try {
	    this.matcher = (FingerprintMatcher) ObjectBuilder.buildObject(fingerprintMatcherName);
	}
	catch (SdTextException e){
	    throw e;
	}
	
    }

    @Override
    public List<Pair<String, Integer>> compute() {

	ArrayList<Pair<String, Integer>> results = new ArrayList<>();
	
	if (end - start <= Global.ForkJoinMinTasks){

	    //System.err.println("In non-recursive branch,matching fingerprints");
	    
	    for (int i = start; i <= end; i++){		

		try {
		    Integer currentScore =
		            matcher.getScore(singleFingerprint.getFingerprint(),
					     fingerprintList.get(i).getFingerprint());

		    results.add(new Pair<>(fingerprintList.get(i).getFileName(), currentScore));
		    
		}
		catch (SdTextException e){
		    // FIX - figure out how to pass exceptions back up here and below
		}				
	    }	    
	}
	else {
	    //System.err.println("In recursive branch, matching fingerprints");
	    
	    try {
		Integer mid = start + ((end - start) / 2);
 		BulkListMatcherWorker left =
		    new BulkListMatcherWorker (fingerprintMatcherName,
					       singleFingerprint,
					       fingerprintList,					 
					       start,
					       mid);

 		BulkListMatcherWorker right =
		    new BulkListMatcherWorker (fingerprintMatcherName,
					       singleFingerprint,
					       fingerprintList,					 
					       mid + 1,
					       end);

		left.fork();
		
		List<Pair<String, Integer>> rightAnswer = right.compute();
		results.addAll(rightAnswer);
		List<Pair<String, Integer>> leftAnswer = left.join();	    
		results.addAll(leftAnswer);	   			       
	    }	    
	    catch (SdTextException e){
		// FIX - figure out how to handle exceptions here, since I can't seem to pass them up
	    }
	    
	}
	
	return results;
    }
}
	
