package edu.georgetown.gucs.tokenizers;

import java.util.Collections;
import java.util.List;
import java.util.Vector;
import java.util.regex.Pattern;

import edu.georgetown.gucs.utility.ExceptionHandler;
import edu.georgetown.gucs.utility.SdTextException;
import edu.georgetown.gucs.utility.Token;

/**
 * Eliminates tokens containing numbers
 *
 * @author Clay Shields
 * @author Lindsay Neubauer
 *
 */
public class RemoveTokensWithNumbersTokenizer extends Tokenizer {

    private static final long serialVersionUID = -5383824450221469920L;
    private static final Pattern containsDigitPattern = Pattern.compile(".*\\d.*");

    /**
     * Eliminates tokens containing numbers
     */
    @Override
    public List<Token> tokenize(List<Token> tokens) throws SdTextException {
        List<Token> tokenVector = Collections.synchronizedList(new Vector<Token>());
        try {
            synchronized (tokenVector) {
                for (Token token : tokens) {
                    if (!containsDigitPattern.matcher(token.name()).matches()) {
                        tokenVector.add(token);
                    }
                }
                return tokenVector;
            }
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
        return tokenVector;
    }
}
