package edu.georgetown.gucs.tokenizers;
import edu.georgetown.gucs.utility.ExceptionHandler;
import edu.georgetown.gucs.utility.Global;
import edu.georgetown.gucs.utility.SdTextException;
import edu.georgetown.gucs.utility.Token;

import java.io.*;
import java.util.List;
import java.util.Vector;
import java.util.logging.Logger;

/**
 * @author Evan B.
 * @since 06/24/14
 */
public class TikaFileTokenizer extends FileTokenizer {

    private static final Logger logger = Logger.getLogger(
            TikaFileTokenizer.class.getName());


    /**
     * Constructor that sets the token creation mode to split based on whitespace
     */
    public TikaFileTokenizer() {
        mode_ = "tokens";
    }

    /**
     * readfile extends FileTokenizer to
     * extract text from non-traditional text files
     *
     * @param fileName the file to extract text from
     * @return a list of Tokens that
     * are extracted from the file
     */

    public List<Token> readFile(String fileName) throws SdTextException {

        try {
            String libPath = Global.getProperty("libPath");

            String params[] = {
                    "java",
                    "-jar",
                    libPath + "/tika-app.jar",
                    "-t",
                    fileName,
            };

            ProcessBuilder ps = new ProcessBuilder(params);
            ps.redirectErrorStream(true);


            try (InputStream inStream = ps.start().getInputStream();
                 BufferedReader buffer = new BufferedReader(
                         new InputStreamReader(inStream, "UTF-8"))) {

                //if the line contains whitespace
                //    parse the line and turn into tokens
                String line;

                //create tempFile to store extracted text
                //this find the location of the token
                File tempFile = File.createTempFile("outsideInOutput", null);
                tempFile.deleteOnExit();

                try (OutputStreamWriter writer = new OutputStreamWriter(
                        new FileOutputStream(tempFile), Global.fileEncoding)) {

                    while ((line = buffer.readLine()) != null) {

                        //write to file
                        writer.write(line);

                    }
                    writer.flush();
                }
                tokenVectorMap.clear();
                Vector<Token> tokenVector = new Vector<>();

                FileTokenizer fileTokenizer = new FileTokenizer(mode_);
                fileTokenizer.tokenize(tempFile.getAbsolutePath());

                tokenVector.clear();
                tokenVector.addAll(fileTokenizer.getTokenVectorMap().get("null"));
                tokenVectorMap.put("null", tokenVector);

            } catch (IOException e) {
                logger.severe("IOException reading File:"
                        + fileName);

                ExceptionHandler.handleException(e);
            }
        } catch(SdTextException sdtext) {
            throw sdtext;
        }
        catch (Exception e) {
            ExceptionHandler.handleException(e);
        }

        return null;
    }
}
