package edu.georgetown.gucs.utility;

import java.util.Arrays;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Prints exception messages and stack traces before calling <code>System.exit(-1)</code>
 *
 * @author Clay Shields
 * @author Lindsay Neubauer
 */
public class ExceptionHandler extends RuntimeException{

    private static final Logger logger = Logger.getLogger(ExceptionHandler.class.getName());

    /**
     * Prints the message and stack trace for this Exception and exits; used with built in java exceptions (not
     * SDText-specific)
     *
     * @param exception the exception that provides information on error
     */
    public static void handleExceptionExit(Exception exception) throws RuntimeException {
        System.err.println("ExceptionHandler:" + exception.getMessage());
        System.err.println("ExceptionHandler " + Arrays.toString(exception.getStackTrace()));
        logger.log(Level.SEVERE, Arrays.toString(exception.getStackTrace()));
        throw new RuntimeException(exception.getMessage());
        //System.exit(-1);
    }

    /**
     * Prints the message and stack trace for this Exception and throws a runtime exception; used with exceptions that are
     * SDText-specific (to give user feedback about error)
     *
     * @param exception the exception that provides information on error
     */
    public static void handleExceptionThrow(Exception exception) {
        logger.log(Level.SEVERE, exception.getMessage());
        throw new RuntimeException(exception.getMessage());
    }


    /**
     * @param status
     */
    public static void safeExit(Integer status) {

        Runtime.getRuntime().exit(status);

    }

    /**
     * @param status
     */
    public static void commandLineError(String status) {

        System.err.println(status);
        Runtime.getRuntime().exit(0);

    }

    /**
     * Prints a string/object with the
     * Calling class/Method
     *
     * @param e
     *
     */
    public static void handleException(Exception e ) throws SdTextException{

        System.setErr(null);
        StackTraceElement[] stackTraceElements = Thread.currentThread().getStackTrace();

        String currentClass = stackTraceElements[2].getFileName();

        String currentMethod = stackTraceElements[2].getMethodName();
        System.setErr(null);
        throw new SdTextException(currentClass + ":" + currentMethod + ":" + e.getMessage());

    }

    @Override
    public synchronized Throwable fillInStackTrace() {

        StackTraceElement[] stackTraceElements = Thread.currentThread().getStackTrace();

        String currentClass = stackTraceElements[2].getFileName();

        String currentMethod = stackTraceElements[2].getMethodName();

        return new SdTextException(currentClass + ":" + currentMethod + ":" );
    }

}
